/* 
    kleds for KDE2

    Copyright (C) 2001 Hans Matzen (hans@hansmatzen.de)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
//
// class for watch the keyboard indicators
//

#include <klocale.h>
#include "xkbwatch.h"

xkbwatch::xkbwatch(char *displaystr) 
    : QObject(0,0), DCOPObject("kledssender")
{
  int major, minor, error_code, status_code;
  displaystring = displaystr;

  // connect to the XServer 
  // get version of XKB
  major = XkbMajorVersion;
  minor = XkbMinorVersion;
  // open connection
  display = XkbOpenDisplay(displaystring, &eventnum,
			   &error_code, &major, &minor, &status_code);

  // check for errors
  if (status_code != XkbOD_Success) {
      if (status_code == XkbOD_BadServerVersion)
	  qWarning(I18N_NOOP("XServer on %s uses wrong XKB version %d.%d"),
		 displaystring, major, minor);

      if (status_code == XkbOD_BadLibraryVersion)
	  qWarning(I18N_NOOP("KLeds is compiled with XKB version %d.%d"),
		 XkbMajorVersion, XkbMinorVersion);
     
      if (status_code == XkbOD_ConnectionRefused)
	  qWarning(I18N_NOOP("Can't open display %s"),displaystring);

      if (status_code == XkbOD_NonXkbServer)
	  qWarning(I18N_NOOP("XKB extension not present"));
  }
  // get the current led state
  if (XkbGetIndicatorState(display, XkbUseCoreKbd, &ledstate) != Success)
      qWarning(I18N_NOOP("Can't get indicator state from Xkb"));

  // remeber the current state
  lednumlock   = ledstate & NUM_LOCK_MASK;
  ledcapslock  = ledstate & CAPS_LOCK_MASK;
  ledscrollock = ledstate & SCROLL_LOCK_MASK;
  
  // configure XKB for only send events on indicators
  XkbSelectEvents(display, XkbUseCoreKbd, 
		  XkbIndicatorStateNotifyMask,XkbIndicatorStateNotifyMask);


  // register with dcopserver
  dc=new DCOPClient();
  
  // attach it
  dc->attach();
  dc->registerAs("kledssender");
}

void xkbwatch::quit() 
{
    this->~xkbwatch();
}

xkbwatch::~xkbwatch() {}

void xkbwatch::run() {
    int ind_state=0;
     
    // emit for the first time to get the current state
    indicators_changed(ledstate);

    // lets loop until we are killed
    while (1) {
	// wait for next X event
	ind_state = waitEvent();
	// send it to the gui
	indicators_changed(ind_state);
	
    }
}

int xkbwatch::waitEvent()
{
    // wait for Event (blocking)
    XNextEvent(display, &xkbevent.core);
    if (xkbevent.core.type == eventnum + XkbEventCode) {
	if (xkbevent.any.xkb_type == XkbIndicatorStateNotify) {
	    
	    // store new state
	    lednumlock   = xkbevent.indicators.state & NUM_LOCK_MASK;
	    ledcapslock  = xkbevent.indicators.state & CAPS_LOCK_MASK;
	    ledscrollock = xkbevent.indicators.state & SCROLL_LOCK_MASK;
	    ledstate = lednumlock | ledcapslock | ledscrollock;
	}
    }
    return ledstate;
}

void xkbwatch::indicators_changed(int ledstate)
{
    
    // send new state via dcop
    QByteArray params;
    QDataStream d(params, IO_WriteOnly);
    d << (int) ledstate;
    
    dc->send("kleds", "kleds", "state_changed(int)",params);
}

void xkbwatch::get_indicators()
{
    indicators_changed(ledstate);
}
